@@ -28,16 +28,17 @@ PerlIOTee_open(pTHX_ PerlIO_funcs* self, PerlIO_list_t* layers, IV n,
 		  const char* mode, int fd, int imode, int perm,
 		  PerlIO* f, int narg, SV** args){
 	PerlIO_funcs* tab = NULL;
+	SV* arg;
 	int i;
+	const char* p;
 
 
-	{
-		const char* p = mode;
-		while(*p++){
-			if(*p == 'r' || *p == '+'){
-				croak("Cannot tee for reading");
-			}
+	p = mode;
+	while(*p){
+		if(*p == 'r' || *p == '+'){
+			Perl_croak(aTHX_ "Cannot tee for reading");
 		}
+		p++;
 	}
 
 	/* find the next layer that has Open() method */
@@ -66,8 +67,9 @@ PerlIOTee_open(pTHX_ PerlIO_funcs* self, PerlIO_list_t* layers, IV n,
 		}
 	}
 
-	if(PerlIOArg && SvOK(PerlIOArg)){
-		if(!PerlIO_push(aTHX_ f, self, mode, PerlIOArg)){
+	arg = PerlIOArg;
+	if(arg && SvOK(arg)){
+		if(!PerlIO_push(aTHX_ f, self, mode, arg)){
 			PerlIO_close(f);
 			return NULL;
 		}
@@ -76,6 +78,7 @@ PerlIOTee_open(pTHX_ PerlIO_funcs* self, PerlIO_list_t* layers, IV n,
 	return f;
 }
 
+static
 IV
 PerlIOTee_pushed(pTHX_ PerlIO* f, const char* mode, SV* arg, PerlIO_funcs* tab){
 	PerlIOTee* t = PerlIOSelf(f, PerlIOTee);
@@ -85,47 +88,49 @@ PerlIOTee_pushed(pTHX_ PerlIO* f, const char* mode, SV* arg, PerlIO_funcs* tab){
 	if(!CanWrite(next)) goto cannot_tee;
 
 	if(SvROK(arg) && (io = GvIO(SvRV(arg)))){
-		t->arg = (SV*)io;
+		t->arg = SvREFCNT_inc_simple_NN( (SV*)io );
 		t->out = IoOFP(io);
+
 		if(!(t->out && CanWrite(t->out))){
 			cannot_tee:
-			SETERRNO(EINVAL, LIB_INVARG);
-			Perl_croak(aTHX_ "Cannot tee for reading");
+			SETERRNO(EBADF, SS_IVCHAN);
+			return -1;
 		}
 	}
 	else{
 		PerlIO_pair_t pairs[] = { { NULL, &PL_sv_undef }, { NULL, &PL_sv_undef } };
 		PerlIO_list_t layers = {
-			1, /* refcnt */
-			0, /* cur */
-			2, /* len */
+			 1, /* refcnt */
+			-1, /* cur */
+			 2, /* len */
 			pairs /* array */
 		};
 		PerlIO_funcs* tab;
-
 		if(SvPOK(arg) && SvCUR(arg) > 2){
 			STRLEN len;
 			const char* pv = SvPV(arg, len);
 			if(*pv == '>'){
 				pv++;
 				len--;
-				if(*pv == '>'){ /* ">>file" */
+				if(*pv == '>'){ /* ">> file" */
 					pv++;
 					len--;
 					mode = "a";
 				}
-				else{ /* ">file" */
+				else{ /* "> file" */
 					mode = "w";
 				}
+				while(isSPACE(*pv)){
+					pv++;
+					len--;
+				}
 			}
 			t->arg = newSVpvn(pv, len);
 		}
 		else{
 			t->arg = newSVsv(arg);
-			if(!mode){
-				mode = "w";
-			}
 		}
+
 		if(SvROK(t->arg) && SvTYPE(SvRV(t->arg)) < SVt_PVAV){
 			tab = PerlIO_find_layer(aTHX_ "scalar", sizeof("scalar"), TRUE);
 			pairs[0].funcs = (PerlIO_funcs*)tab; /* const_cast */
@@ -139,16 +144,17 @@ PerlIOTee_pushed(pTHX_ PerlIO* f, const char* mode, SV* arg, PerlIO_funcs* tab){
 			layers.cur = 2;
 
 		}
+		if(!mode){
+			mode = "w";
+		}
+
+		PerlIO_debug("PerlIOTee: %s => %s\n", tab->name, SvPV_nolen_const(t->arg));
 
 		t->out = tab->Open(aTHX_ tab, &layers,
 			layers.cur, mode, -1, 0, 0, NULL, 1, &(t->arg));
 	}
-	if(!t->out && ckWARN(WARN_IO)){
-		Perl_warner(aTHX_ packWARN(WARN_IO),
-			"Cannot open '%s': %s",
-			SvOK(t->arg) ? SvPV_nolen(t->arg) : "",
-			Strerror(errno));
-		return -1;
+	if(!PerlIOValid(t->out)){
+		return -1; /* failure */
 	}
 
 	PerlIOBase(f)->flags = PerlIOBase(t->out)->flags = PerlIOBase(next)->flags;
@@ -156,6 +162,7 @@ PerlIOTee_pushed(pTHX_ PerlIO* f, const char* mode, SV* arg, PerlIO_funcs* tab){
 	return 0;
 }
 
+static
 IV
 PerlIOTee_popped(pTHX_ PerlIO* f){
 	PerlIOTee* t = PerlIOSelf(f, PerlIOTee);
@@ -170,6 +177,7 @@ PerlIOTee_popped(pTHX_ PerlIO* f){
 	return 0;
 }
 
+static
 SV*
 PerlIOTee_getarg(pTHX_ PerlIO* f, CLONE_PARAMS* param, int flags){
 	PerlIOTee* t = PerlIOSelf(f, PerlIOTee);
@@ -177,6 +185,7 @@ PerlIOTee_getarg(pTHX_ PerlIO* f, CLONE_PARAMS* param, int flags){
 	return PerlIO_sv_dup(aTHX_ t->arg, param);
 }
 
+static
 SSize_t
 PerlIOTee_write(pTHX_ PerlIO* f, const void* vbuf, Size_t count){
 	PerlIOTee* t = PerlIOSelf(f, PerlIOTee);
@@ -189,6 +198,7 @@ PerlIOTee_write(pTHX_ PerlIO* f, const void* vbuf, Size_t count){
 	return PerlIO_write(next, vbuf, count);
 }
 
+static
 IV
 PerlIOTee_flush(pTHX_ PerlIO* f){
 	PerlIOTee* t = PerlIOSelf(f, PerlIOTee);
@@ -201,6 +211,25 @@ PerlIOTee_flush(pTHX_ PerlIO* f){
 	return PerlIO_flush(next);
 }
 
+static
+IV
+PerlIOTee_seek(pTHX_ PerlIO* f, Off_t offset, int whence){
+	PerlIOTee* t = PerlIOSelf(f, PerlIOTee);
+	PerlIO* next = PerlIONext(f);
+	IV code;
+
+	if((code = PerlIOTee_flush(aTHX_ f)) == 0){
+		if(PerlIO_seek(t->out, offset, whence) != 0){
+			/* warn? */
+		}
+
+		code = PerlIO_seek(next, offset, whence);
+	}
+
+	return code;
+}
+
+static
 Off_t
 PerlIOTee_tell(pTHX_ PerlIO* f){
 	PerlIO* next = PerlIONext(f);
@@ -219,20 +248,20 @@ PERLIO_FUNCS_DECL(PerlIO_tee) = {
     PerlIOTee_open,
     PerlIOBase_binmode,
     PerlIOTee_getarg,
-    PerlIOBase_fileno,
+    NULL, /* fileno */
     PerlIOBase_dup,
     NULL, /* read */
     NULL, /* unread */
     PerlIOTee_write,
-    NULL, /* seek */
+    PerlIOTee_seek,
     PerlIOTee_tell,
     NULL, /* close */
     PerlIOTee_flush,
     NULL, /* fill */
     NULL, /* eof */
-    PerlIOBase_error,
-    PerlIOBase_clearerr,
-    PerlIOBase_setlinebuf,
+    NULL, /* error */
+    NULL, /* clearerror */
+    NULL, /* setlinebuf */
     NULL, /* get_base */
     NULL, /* bufsiz */
     NULL, /* get_ptr */
